body
{
    font-size:   14px;
    line-height: 1.6;
    margin:      0 auto;
    max-width:   960px;
}

p code
{
    background:    #f2f2f2;
    padding-left:  3px;
    padding-right: 3px;
}

pre.code
{
    font-size:   13px;
    line-height: 1.4;
}

/**
 * YARD uses generic table styles, using a special class means those tables
 * don't get messed up.
 */
.table
{
    border:          1px solid #ccc;
    border-right:    none;
    border-collapse: separate;
    border-spacing:  0;
    text-align:      left;
}

.table.full
{
    width: 100%;
}

    .table .field_name
    {
        min-width: 160px;
    }

    .table thead tr th.no_sort:first-child
    {
        width: 25px;
    }

    .table thead tr th, .table tbody tr td
    {
        border-bottom:  1px solid #ccc;
        border-right:   1px solid #ccc;
        min-width:      20px;
        padding:        8px 5px;
        text-align:     left;
        vertical-align: top;
    }

    .table tbody tr:last-child td
    {
        border-bottom: none;
    }

    .table tr:nth-child(odd) td
    {
        background: #f9f9f9;
    }
